export function debounce(func, { waitMs, timing = "trailing", maxWaitMs, }) {
    if (maxWaitMs !== undefined && waitMs !== undefined && maxWaitMs < waitMs) {
        throw new Error(`debounce: maxWaitMs (${maxWaitMs}) cannot be less than waitMs (${waitMs})`);
    }
    let coolDownTimeoutId;
    let maxWaitTimeoutId;
    let latestCallArgs;
    let result;
    const handleInvoke = () => {
        if (latestCallArgs === undefined) {
            return;
        }
        if (maxWaitTimeoutId !== undefined) {
            const timeoutId = maxWaitTimeoutId;
            maxWaitTimeoutId = undefined;
            clearTimeout(timeoutId);
        }
        const args = latestCallArgs;
        latestCallArgs = undefined;
        result = func(...args);
    };
    const handleCoolDownEnd = () => {
        if (coolDownTimeoutId === undefined) {
            return;
        }
        const timeoutId = coolDownTimeoutId;
        coolDownTimeoutId = undefined;
        clearTimeout(timeoutId);
        if (latestCallArgs !== undefined) {
            handleInvoke();
        }
    };
    const handleDebouncedCall = (args) => {
        latestCallArgs = args;
        if (maxWaitMs !== undefined && maxWaitTimeoutId === undefined) {
            maxWaitTimeoutId = setTimeout(handleInvoke, maxWaitMs);
        }
    };
    return {
        call: (...args) => {
            if (coolDownTimeoutId === undefined) {
                if (timing === "trailing") {
                    handleDebouncedCall(args);
                }
                else {
                    result = func(...args);
                }
            }
            else {
                if (timing !== "leading") {
                    handleDebouncedCall(args);
                }
                const timeoutId = coolDownTimeoutId;
                coolDownTimeoutId = undefined;
                clearTimeout(timeoutId);
            }
            coolDownTimeoutId = setTimeout(handleCoolDownEnd, waitMs ?? maxWaitMs ?? 0);
            return result;
        },
        cancel: () => {
            if (coolDownTimeoutId !== undefined) {
                const timeoutId = coolDownTimeoutId;
                coolDownTimeoutId = undefined;
                clearTimeout(timeoutId);
            }
            if (maxWaitTimeoutId !== undefined) {
                const timeoutId = maxWaitTimeoutId;
                maxWaitTimeoutId = undefined;
                clearTimeout(timeoutId);
            }
            latestCallArgs = undefined;
        },
        flush: () => {
            handleCoolDownEnd();
            return result;
        },
        get isPending() {
            return coolDownTimeoutId !== undefined;
        },
        get cachedValue() {
            return result;
        },
    };
}
